Package: interleave
Type: Package
Title: Converts Tabular Data to Interleaved Vectors
Version: 0.1.2
Date: 2024-01-18
Authors@R: c(
        person("David", "Cooley", email = "david.cooley.au@gmail.com", role = c("aut", "cre")), 
        person("Mapbox", role = "cph", comment = "author of header library earcut.hpp")
        )
Description: Converts matrices and lists of matrices into a single vector by interleaving 
    their values. That is, each element of the result vector is filled from the input 
    matrices one row at a time. This is the same as transposing a matrix, then removing the 
    dimension attribute, but is designed to operate on matrices in nested list structures.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.2.3
Depends: R (>= 3.0.2)
LinkingTo: geometries (>= 0.2.4), Rcpp
Imports: Rcpp
Suggests: covr, sfheaders, tinytest
NeedsCompilation: yes
Packaged: 2024-01-17 23:38:07 UTC; dave
Author: David Cooley [aut, cre],
  Mapbox [cph] (author of header library earcut.hpp)
Maintainer: David Cooley <david.cooley.au@gmail.com>
Repository: CRAN
Date/Publication: 2024-01-17 23:50:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 20:19:45 UTC; unix
Archs: interleave.so.dSYM
